import os
import core.progressWindowManager as progressWindowManager
from characterConfig import CharacterConfig

from modules.dna.model.gui import AnalogGuiOptions
from modules.dna.scene.builder.rlNodeBuilder import RigLogicNodeBuilder
from modules.file.handler import FileHandler
from modules.dna.scene.builder.sceneBuilder import SceneBuilder, SceneBuilderError
import pymel.core as pycore
import maya.cmds as cmds


class DHIImporterSetup:
    Instance = None

    def __init__(self):
        DHIImporterSetup.Instance = self

    def set_Asset_Data(self, json_data):
        print("Received JSON data for Digital Human Asset")
        self.jsonData = json_data
        self.characterConfig = CharacterConfig(self.jsonData["character"])
        print self.characterConfig
        confirmDialogResponse = pycore.confirmDialog(title='Confirm dialog',
                                                     message='Are you sure you want to import MetaHuman character? All unsaved progress will be lost.',
                                                     button=['Yes', 'No'],
                                                     defaultButton='Yes',
                                                     cancelButton='No', dismissString='No')

        if confirmDialogResponse == 'Yes':
            self.importCharacter()

    def importCharacter(self):
        try:
            import time
            start_time = time.time()
            progressWindowManager.reset()
            progressWindowManager.show()
            self.preImportCharacter()

            self.importHead()
            self.importBodyAndAttachToHead()

            self.postImportCharacter()
            progressWindowManager.hide()
            print ("--- Import finished in %s seconds ---" % (time.time() - start_time))
        except:
            progressWindowManager.hide()

    def preImportCharacter(self):
        self.backupDNAWithGeometry()

    def backupDNAWithGeometry(self):
        from shutil import copyfile
        rlDNAPath = self.resolveRLDNAPathFromDNAPath(self.characterConfig.dnaPath)
        copyfile(self.characterConfig.dnaPath, rlDNAPath)

    def resolveRLDNAPathFromDNAPath(self, dnaPath):
        '''
        returns a string with _rl as a sufix
        '''
        return dnaPath.replace('.dna', '') + "_rl.dna"

    def postImportCharacter(self):
        print("Post import reached")
        from DHI.modules.maya.util import MayaUtil
        MayaUtil.removeGeometryFromDNA(self.resolveRLDNAPathFromDNAPath(self.characterConfig.dnaPath))

        # remove sufficient meshes
        resolvedBodyMeshName = self.characterConfig.bodyMeshName.replace("_body", "")
        print("resolvedBodyMeshName", resolvedBodyMeshName)

        print("Removing sufficient objects")
        if pycore.objExists("ROM_root"):
            pycore.delete("ROM_root")

        if pycore.objExists(resolvedBodyMeshName + "_lod0"):
            pycore.delete(resolvedBodyMeshName + "_lod0")

        if pycore.objExists(resolvedBodyMeshName + "_lod1"):
            pycore.delete(resolvedBodyMeshName + "_lod1")

        if pycore.objExists(resolvedBodyMeshName + "_lod2"):
            pycore.delete(resolvedBodyMeshName + "_lod2")

        if pycore.objExists(resolvedBodyMeshName + "_lod3"):
            pycore.delete(resolvedBodyMeshName + "_lod3")

        if pycore.objExists(resolvedBodyMeshName + "_combined_lod0_mesh"):
            pycore.delete(resolvedBodyMeshName + "_combined_lod0_mesh")

        if pycore.objExists(resolvedBodyMeshName + "_body_uExport"):
            pycore.delete(resolvedBodyMeshName + "_body_uExport")

        print("Moving body to world space")
        pycore.parent("root", world=True)
        pycore.parent("root_drv", world=True)

        if pycore.objExists("Skeletons"):
            pycore.delete("Skeletons")

        print("Adding body namespace")
        bodyNamespace = "DHIbody"
        cmds.select("root")
        bodyJointNames = cmds.listRelatives(allDescendents=True, type='joint')
        bodyJointNames.append("root")
        bodyJoints = []
        for mayaJoint in bodyJointNames:
            # get joint node
            jointNodes = pycore.ls(mayaJoint, type="joint")
            bodyJoints.append(jointNodes[0])

        self.createControlsSet()
        self.createBodyJointsSet(bodyJointNames)

        pycore.namespace(add=bodyNamespace)
        self.addJointsToNamespace(bodyNamespace, bodyJoints)

        # remove animation joint chain0
        if pycore.objExists("anim:root"):
            pycore.delete("anim:root")

    def createBodyJointsSet(self, bodyJoints):
        pycore.select(clear=True)
        for bj in bodyJoints:
            if pycore.objExists(bj):
                cmds.select(bj, add=True)
        cmds.sets(n="Body joints")

    def createControlsSet(self):
        import dna
        fileStream = dna.FileStream(str(self.characterConfig.dnaPath),
                                    dna.FileStream.AccessMode_Read, dna.FileStream.OpenMode_Binary)
        reader = dna.StreamReader(fileStream)
        reader.read()
        pycore.select(clear=True)
        for i in range(reader.getGUIControlCount()):
            controlName = reader.getGUIControlName(i).split(".")[0]
            if pycore.objExists(controlName):
                cmds.select(controlName, add=True)

        if pycore.objExists("CTRL_C_eye"):
            cmds.select("CTRL_C_eye", add=True)

        if pycore.objExists("CTRL_rigLogicSwitch"):
            cmds.select("CTRL_rigLogicSwitch", add=True)

        if pycore.objExists("CTRL_lookAtSwitchShape"):
            cmds.select("CTRL_lookAtSwitchShape", add=True)

        if pycore.objExists("CTRL_C_eyesAim"):
            cmds.select("CTRL_C_eyesAim", add=True)

        if pycore.objExists("CTRL_C_neck_swallowShape"):
            cmds.select("CTRL_C_neck_swallowShape", add=True)

        if pycore.objExists("CTRL_L_eyeAim"):
            cmds.select("CTRL_L_eyeAim", add=True)

        if pycore.objExists("CTRL_R_eyeAim"):
            cmds.select("CTRL_R_eyeAim", add=True)

        if pycore.objExists("CTRL_convergenceSwitch"):
            cmds.select("CTRL_convergenceSwitch", add=True)

        cmds.sets(n="Controls")

    def moveBodyMeshesToLODGroups(self, bodyMeshName):
        try:
            bodyMeshes = []
            distances = [100, 500, 1500, 3000]

            # move body meshes to world so we can group them in LOD (no parents allowed)
            for i in range(0, 4):
                meshName = "%s_lod%s_mesh" % (bodyMeshName, str(i))
                pycore.parent(meshName, world=True)
                bodyMeshes.append(meshName)

            # create LOD group
            pycore.select(bodyMeshes, replace=True)
            pycore.mel.eval("LevelOfDetailGroup;")
            pycore.rename("LOD_Group_1", "Body_Lx")
            pycore.setAttr("Body_Lx.useScreenHeightPercentage", 0)  # Switch to distance
            pycore.parent("Body_Lx", "geometry_grp")

            # set LOD threshold based on distances
            for i, val in enumerate(bodyMeshes):
                pycore.setAttr("Body_Lx.threshold[%s]" % i, distances[i])

            # remove unused body group and meshes
            if pycore.objExists("export_geo_grp"):
                pycore.delete("export_geo_grp")
        except Exception as e:
            print("Error moving body meshes to their corresponding LOD groups")
            print e
            pass

    def importBodyAndAttachToHead(self):
        from DHI.modules.file.reader import PythonConfigurationReader
        current = os.path.dirname(os.path.abspath(__file__))

        progressWindowManager.update(0, 'Starting body import')

        self.configurationModule = PythonConfigurationReader(
            FileHandler.joinPath((current, "core/consts.py"))).readFile()

        # MayaUtil.createHeadAndBodyScene(self.configurationModule.MESH_NAMES,
        #                                 self.configurationModule.FACIAL_ROOT_JOINTS,
        #                                 self.configurationModule.NECK_JOINTS,
        #                                 self.characterConfig.bodyScenePath)

        self.importBodyAndParentConstraint()
        self.removeUnknownPlugins()
        self.moveBodyMeshesToLODGroups(self.characterConfig.bodyMeshName)
        progressWindowManager.update(0, 'Body import done')

    def removeUnknownPlugins(self):
        unknown_plugins = cmds.unknownPlugin(query=True, list=True)
        if unknown_plugins:
            for plugin in unknown_plugins:
                try:
                    cmds.unknownPlugin(plugin, remove=True)
                except Exception as err:
                    print(err.message)

    def importBodyAndParentConstraint(self):
        headNamespace = "DHIhead"
        headJoints = [joint for joint in pycore.ls(type="joint")]

        pycore.namespace(add=headNamespace)
        self.addJointsToNamespace(headNamespace, headJoints)

        # import body to scene
        pycore.importFile(self.characterConfig.bodyScenePath, options="v=0", type="mayaAscii")

        # add parent constraint for all matching head joints to body joints
        cmds.select("root")
        bodyJoints = cmds.listRelatives(allDescendents=True, type='joint')
        bodyJoints.append("root")
        for bj in bodyJoints:
            hj = headNamespace + ":" + bj
            if hj in headJoints:
                pycore.parentConstraint(bj, hj, maintainOffset=True)
                pycore.scaleConstraint(bj, hj, maintainOffset=True)

    def addJointsToNamespace(self, name, joints):
        for joint in joints:
            newname = joint.swapNamespace(name)
            joint.rename(newname)

    def importHeadShaders(self):
        from DHI.modules.maya.util import MayaUtil
        from DHI.core.consts import MASKS, SHADERS, MESH_SHADER_MAPPING, MAP_INFOS, COMMON_MAP_INFOS

        shaderScenePath = self.characterConfig.shaderScenePath
        print("Shader scene path:", shaderScenePath)
        progressWindowManager.update(0, "Importing shader scene from: %s" % shaderScenePath)
        MayaUtil.importShader(shaderScenePath, MESH_SHADER_MAPPING)
        MayaUtil.resolveSceneShaderPaths(SHADERS, self.characterConfig.shadersDirPath)
        MayaUtil.resolveSceneMaskPaths(MASKS, self.characterConfig.masksDirPath)
        MayaUtil.resolveSceneMapPaths(COMMON_MAP_INFOS, self.characterConfig.headMapsPath)
        MayaUtil.resolveSceneMapPaths(MAP_INFOS, self.characterConfig.mapsDirPath)

        progressWindowManager.update(0, "Shader imported.")

    def importHeadLightScene(self):
        progressWindowManager.update(0, "Importing head light scene.")
        from DHI.modules.maya.util import MayaUtil

        lightScenePath = self.characterConfig.lightScenePath
        MayaUtil.createLights(lightScenePath, self.characterConfig.sceneOrientation)
        progressWindowManager.update(0, "Importing head light scene done.")

    def importHead(self):
        progressWindowManager.update(0, 'Starting head import')

        dnaPath = self.resolveRLDNAPathFromDNAPath(self.characterConfig.dnaPath)
        guiPath = self.characterConfig.guiPath
        progressWindowManager.update(0, "Importing head from DNA path: %s" % dnaPath)

        pycore.newFile(force=True)
        pycore.upAxis(ax=self.characterConfig.sceneOrientationStringValue, rv=True)

        characterName = FileHandler.getCharNameFromFileName(dnaPath)[0]

        rl4nb = RigLogicNodeBuilder()
        rl4nb.setName("rl4Embedded_" + characterName)
        rl4nb.setFlag("-dfp", dnaPath)

        analogGuiOptions = AnalogGuiOptions.defaultOptions()
        analogGuiOptions.GuiPath = self.characterConfig.acPath
        current = os.path.dirname(os.path.abspath(__file__))

        sceneBuilder = SceneBuilder()

        sceneBuilder. \
            FromDnaPath(dnaPath). \
            And(). \
            WithGui(guiPath). \
            And(). \
            WithAnalogGuiOptions(analogGuiOptions). \
            And(). \
            WithFRMAttrs("FRM_WMmultipliers"). \
            And(). \
            WithRigLogicNodeBuilder(rl4nb). \
            And(). \
            WithNormals(). \
            And(). \
            WithSceneOrientation(self.characterConfig.sceneOrientation). \
            And(). \
            WithLinearUnit("cm"). \
            And(). \
            WithAngleUnit("degree"). \
            And(). \
            WithAAS(FileHandler.joinPath((current, "connectexp.py")))

        try:
            sceneBuilder.build()
            self.importHeadShaders()
            self.importHeadLightScene()
        except SceneBuilderError as ex:
            print "Scene builder error", ex.message
        except Exception as ex:
            print "Unexpected 0error occurred - %s!" % ex.message
            print ex
